@@ -12,7 +12,14 @@ from tempfile import TemporaryDirectory
 from typing import Callable, Dict, List, Optional, Tuple, Union, Any, Generator
 from pathlib import Path
 import numpy as np
-import numpy.typing
+
+# We import the typing under try-catch to allow runtime for systems that have
+# old Numpy that don't feature the numpy.typing module, but we want to preserve
+# type checking.
+try:
+    import numpy.typing
+except ImportError:
+    pass
 
 from PIL import Image, ImageChops, ImageDraw, ImageFilter
 from pyvirtualdisplay.smartdisplay import SmartDisplay
